@@ -1667,10 +1667,7 @@ long cm_BPlusDirCreateEntry(cm_dirOp_t * op, char *entry, cm_fid_t * cfid)
 
     lock_AssertWrite(&op->scp->dirlock);
 
-    data.fid.cell = cfid->cell;
-    data.fid.volume = cfid->volume;
-    data.fid.vnode = cfid->vnode;
-    data.fid.unique = cfid->unique;
+    cm_SetFid(&data.fid, cfid->cell, cfid->volume, cfid->vnode, cfid->unique);
     data.longname = NULL;
 
     QueryPerformanceCounter(&start);
@@ -1857,10 +1854,7 @@ int cm_BPlusDirFoo(struct cm_scache *scp, struct cm_dirEntry *dep,
     dataT  data;
     char   shortName[13];
 
-    data.fid.cell = scp->fid.cell;
-    data.fid.volume = scp->fid.volume;
-    data.fid.vnode = ntohl(dep->fid.vnode);
-    data.fid.unique = ntohl(dep->fid.unique);
+    cm_SetFid(&data.fid, scp->fid.cell, scp->fid.volume, ntohl(dep->fid.vnode), ntohl(dep->fid.unique));
     data.longname = NULL;
 
     /* the Write lock is held in cm_BPlusDirBuildTree() */
@@ -2185,7 +2179,7 @@ cm_BPlusDirEnumTest(cm_scache_t * dscp, afs_uint32 locked)
 	    char buffer[1024];
 	    cm_scache_t *scp;
 	    char * type = "ScpNotFound";
-	    afs_int32 dv = -1;
+	    afs_uint64 dv = -1;
 
 	    scp = cm_FindSCache(&entryp->fid);
 	    if (scp) {
@@ -2214,10 +2208,10 @@ cm_BPlusDirEnumTest(cm_scache_t * dscp, afs_uint32 locked)
 		}
 
 		dv = scp->dataVersion;
-	    cm_ReleaseSCache(scp);
+                cm_ReleaseSCache(scp);
 	    }
 
-	    sprintf(buffer, "'%s' Fid = (%d,%d,%d,%d) Short = '%s' Type %s DV %d",
+	    sprintf(buffer, "'%s' Fid = (%d,%d,%d,%d) Short = '%s' Type %s DV %I64d",
 		    entryp->name,
 		    entryp->fid.cell, entryp->fid.volume, entryp->fid.vnode, entryp->fid.unique,
 		    entryp->shortName,
