@@ -28,6 +28,7 @@ osi_rwlock_t cm_connLock;
 DWORD RDRtimeout = CM_CONN_DEFAULTRDRTIMEOUT;
 unsigned short ConnDeadtimeout = CM_CONN_CONNDEADTIME;
 unsigned short HardDeadtimeout = CM_CONN_HARDDEADTIME;
+unsigned short IdleDeadtimeout = CM_CONN_IDLEDEADTIME;
 
 #define LANMAN_WKS_PARAM_KEY "SYSTEM\\CurrentControlSet\\Services\\lanmanworkstation\\parameters"
 #define LANMAN_WKS_SESSION_TIMEOUT "SessTimeout"
@@ -86,7 +87,15 @@ void cm_InitConn(void)
 	    if (code == ERROR_SUCCESS)
                 HardDeadtimeout = (unsigned short)dwValue;
 	    afsi_log("HardDeadTimeout is %d", HardDeadtimeout);
-	    RegCloseKey(parmKey);
+
+	    dummyLen = sizeof(DWORD);
+	    code = RegQueryValueEx(parmKey, "IdleDeadTimeout", NULL, NULL,
+				    (BYTE *) &dwValue, &dummyLen);
+	    if (code == ERROR_SUCCESS)
+                IdleDeadtimeout = (unsigned short)dwValue;
+	    afsi_log("IdleDeadTimeout is %d", IdleDeadtimeout);
+
+            RegCloseKey(parmKey);
 	}
 
 	afsi_log("lanmanworkstation : SessTimeout %u", RDRtimeout);
@@ -133,7 +142,9 @@ static long cm_GetServerList(struct cm_fid *fidp, struct cm_user *userp,
     
     *serversppp = cm_GetVolServers(volp, fidp->volume);
 
+    lock_ObtainRead(&cm_volumeLock);
     cm_PutVolume(volp);
+    lock_ReleaseRead(&cm_volumeLock);
     return 0;
 }
 
@@ -172,7 +183,7 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
     int dead_session;
     long timeUsed, timeLeft;
     long code;
-    char addr[16];
+    char addr[16]="unknown";
     int forcing_new = 0;
 
     osi_Log2(afsd_logp, "cm_Analyze connp 0x%p, code 0x%x",
@@ -272,7 +283,8 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
     else if (errorCode == CM_ERROR_ALLDOWN) {
 	osi_Log0(afsd_logp, "cm_Analyze passed CM_ERROR_ALLDOWN.");
 	/* Servers marked DOWN will be restored by the background daemon
-	 * thread as they become available.
+	 * thread as they become available.  The volume status is 
+         * updated as the server state changes.
 	 */
     }
 
@@ -281,55 +293,45 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
         /* Volume instances marked offline will be restored by the 
          * background daemon thread as they become available 
          */
-        if (timeLeft > 7 && fidp) {
-            thrd_Sleep(5000);
-
+        if (fidp) {
             code = cm_FindVolumeByID(cellp, fidp->volume, userp, reqp, 
-                                    CM_GETVOL_FLAG_NO_LRU_UPDATE, 
-                                    &volp);
+                                      CM_GETVOL_FLAG_NO_LRU_UPDATE, 
+                                      &volp);
             if (code == 0) {
-                if (fidp->volume == volp->rw.ID)
-                    statep = &volp->rw;
-                else if (fidp->volume == volp->ro.ID)
-                    statep = &volp->ro;
-                else if (fidp->volume == volp->bk.ID)
-                    statep = &volp->bk;
+                if (timeLeft > 7) {
+                    thrd_Sleep(5000);
 
-                if (statep->state != vl_offline && statep->state != vl_unknown) {
-                    retry = 1;
-                } else {
-                    if (cm_CheckOfflineVolume(volp, statep->ID))
+                    /* cm_CheckOfflineVolume() resets the serverRef state */
+                    if (cm_CheckOfflineVolume(volp, fidp->volume))
                         retry = 1;
+                } else {
+                    cm_UpdateVolumeStatus(volp, fidp->volume);
                 }
-            
+                lock_ObtainRead(&cm_volumeLock);
                 cm_PutVolume(volp);
+                lock_ReleaseRead(&cm_volumeLock);
+                volp = NULL;
             }
-        }
+        } 
     }
     else if (errorCode == CM_ERROR_ALLBUSY) {
         /* Volumes that are busy cannot be determined to be non-busy 
          * without actually attempting to access them.
          */
 	osi_Log0(afsd_logp, "cm_Analyze passed CM_ERROR_ALLBUSY.");
-        if (timeLeft > 7) {
-
-            thrd_Sleep(5000);
-
-            if (fidp) { /* File Server query */
-                code = cm_FindVolumeByID(cellp, fidp->volume, userp, reqp, 
-                                        CM_GETVOL_FLAG_NO_LRU_UPDATE, 
-                                        &volp);
-                if (code == 0) {
-                    if (fidp->volume == volp->rw.ID)
-                        statep = &volp->rw;
-                    else if (fidp->volume == volp->ro.ID)
-                        statep = &volp->ro;
-                    else if (fidp->volume == volp->bk.ID)
-                        statep = &volp->bk;
 
+        if (fidp) { /* File Server query */
+            code = cm_FindVolumeByID(cellp, fidp->volume, userp, reqp, 
+                                     CM_GETVOL_FLAG_NO_LRU_UPDATE, 
+                                     &volp);
+            if (code == 0) {
+                if (timeLeft > 7) {
+                    thrd_Sleep(5000);
+                    
+                    statep = cm_VolumeStateByID(volp, fidp->volume);
                     if (statep->state != vl_offline && 
-                        statep->state != vl_busy &&
-                        statep->state != vl_unknown) {
+                         statep->state != vl_busy &&
+                         statep->state != vl_unknown) {
                         retry = 1;
                     } else {
                         if (!serversp) {
@@ -343,7 +345,7 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
                         for (tsrp = serversp; tsrp; tsrp=tsrp->next) {
                             if (tsrp->status == srv_busy) {
                                 tsrp->status = srv_not_busy;
-                            }
+                            }       
                         }
                         lock_ReleaseWrite(&cm_serverLock);
                         if (free_svr_list) {
@@ -351,13 +353,22 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
                             *serverspp = serversp;
                         }
 
-                        cm_UpdateVolumeStatus(volp, statep->ID);
+                        cm_UpdateVolumeStatus(volp, fidp->volume);
                         retry = 1;
                     }
-            
-                    cm_PutVolume(volp);
+                } else {
+                    cm_UpdateVolumeStatus(volp, fidp->volume);
                 }
-            } else {    /* VL Server query */
+
+                lock_ObtainRead(&cm_volumeLock);
+                cm_PutVolume(volp);
+                lock_ReleaseRead(&cm_volumeLock);
+                volp = NULL;
+            }
+        } else {    /* VL Server query */
+            if (timeLeft > 7) {
+                thrd_Sleep(5000);
+
                 if (serversp) {
                     lock_ObtainWrite(&cm_serverLock);
                     for (tsrp = serversp; tsrp; tsrp=tsrp->next) {
@@ -389,14 +400,8 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
                     code = cm_FindVolumeByID(cellp, fidp->volume, userp, reqp, 
                                              CM_GETVOL_FLAG_NO_LRU_UPDATE, 
                                              &volp);
-                    if (code == 0) {
-                        if (fidp->volume == volp->rw.ID)
-                            statep = &volp->rw;
-                        else if (fidp->volume == volp->ro.ID)
-                            statep = &volp->ro;
-                        else if (fidp->volume == volp->bk.ID)
-                            statep = &volp->bk;
-                    }
+                    if (code == 0)
+                        statep = cm_VolumeStateByID(volp, fidp->volume);
                 }
                 break;
             }
@@ -405,7 +410,10 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
         
         if (statep) {
             cm_UpdateVolumeStatus(volp, statep->ID);
+            lock_ObtainRead(&cm_volumeLock);
             cm_PutVolume(volp);
+            lock_ReleaseRead(&cm_volumeLock);
+            volp = NULL;
         }
 
         if (free_svr_list) {
@@ -500,14 +508,8 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
                     code = cm_FindVolumeByID(cellp, fidp->volume, userp, reqp, 
                                              CM_GETVOL_FLAG_NO_LRU_UPDATE, 
                                              &volp);
-                    if (code == 0) {
-                        if (fidp->volume == volp->rw.ID)
-                            statep = &volp->rw;
-                        else if (fidp->volume == volp->ro.ID)
-                            statep = &volp->ro;
-                        else if (fidp->volume == volp->bk.ID)
-                            statep = &volp->bk;
-                    }
+                    if (code == 0)
+                        cm_VolumeStateByID(volp, fidp->volume);
                 }   
             }
         }   
@@ -518,7 +520,10 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
 
         if (statep) {
             cm_UpdateVolumeStatus(volp, statep->ID);
+            lock_ObtainRead(&cm_volumeLock);
             cm_PutVolume(volp);
+            lock_ReleaseRead(&cm_volumeLock);
+            volp = NULL;
         }
 
         if (free_svr_list) {
@@ -540,7 +545,6 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
 		if (scp->fileType != CM_SCACHETYPE_DIRECTORY)
 		    pscp = cm_FindSCacheParent(scp);
 
-
 		lock_ObtainWrite(&scp->rw);
 		lock_ObtainWrite(&cm_scacheLock);
 		cm_RemoveSCacheFromHashTable(scp);
@@ -572,20 +576,24 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
          * In addition, we log an event to the event log 
          */
 
-        /* Log server being offline for this volume */
-        sprintf(addr, "%d.%d.%d.%d", 
-                 ((serverp->addr.sin_addr.s_addr & 0xff)),
-                 ((serverp->addr.sin_addr.s_addr & 0xff00)>> 8),
-                 ((serverp->addr.sin_addr.s_addr & 0xff0000)>> 16),
-                 ((serverp->addr.sin_addr.s_addr & 0xff000000)>> 24)); 
+        if (serverp) {
+            /* Log server being offline for this volume */
+            sprintf(addr, "%d.%d.%d.%d", 
+                     ((serverp->addr.sin_addr.s_addr & 0xff)),
+                     ((serverp->addr.sin_addr.s_addr & 0xff00)>> 8),
+                     ((serverp->addr.sin_addr.s_addr & 0xff0000)>> 16),
+                     ((serverp->addr.sin_addr.s_addr & 0xff000000)>> 24)); 
 
 #ifndef DJGPP
-	LogEvent(EVENTLOG_WARNING_TYPE, MSG_RX_HARD_DEAD_TIME_EXCEEDED, addr);
+            LogEvent(EVENTLOG_WARNING_TYPE, MSG_RX_HARD_DEAD_TIME_EXCEEDED, addr);
 #endif /* !DJGPP */
 	  
-        retry = 0;
-        osi_Log1(afsd_logp, "cm_Analyze: hardDeadTime exceeded addr[%s]",
-		 osi_LogSaveString(afsd_logp,addr));
+            osi_Log1(afsd_logp, "cm_Analyze: hardDeadTime exceeded addr[%s]",
+                     osi_LogSaveString(afsd_logp,addr));
+            reqp->tokenIdleErrorServp = serverp;
+            reqp->idleError++;
+            retry = 1;
+        }
     }
     else if (errorCode >= -64 && errorCode < 0) {
         /* mark server as down */
@@ -619,9 +627,11 @@ cm_Analyze(cm_conn_t *connp, cm_user_t *userp, cm_req_t *reqp,
                 retry = 1;
         }
     } else if (errorCode >= ERROR_TABLE_BASE_RXK && errorCode < ERROR_TABLE_BASE_RXK + 256) {
-        reqp->tokenErrorServp = serverp;
-        reqp->tokenError = errorCode;
-        retry = 1;
+        if (serverp) {
+            reqp->tokenIdleErrorServp = serverp;
+            reqp->tokenError = errorCode;
+            retry = 1;
+        }
     } else if (errorCode == VICECONNBAD || errorCode == VICETOKENDEAD) {
 	cm_ForceNewConnections(serverp);
         if ( timeLeft > 2 )
@@ -809,15 +819,15 @@ long cm_ConnByMServers(cm_serverRef_t *serversp, cm_user_t *usersp,
     lock_ObtainRead(&cm_serverLock);
     for (tsrp = serversp; tsrp; tsrp=tsrp->next) {
         tsp = tsrp->server;
-        if (reqp->tokenErrorServp) {
+        if (reqp->tokenIdleErrorServp) {
             /* 
              * search the list until we find the server
              * that failed last time.  When we find it
              * clear the error, skip it and try the one
              * in the list.
              */
-            if (tsp == reqp->tokenErrorServp)
-                reqp->tokenErrorServp = NULL;
+            if (tsp == reqp->tokenIdleErrorServp)
+                reqp->tokenIdleErrorServp = NULL;
             continue;
         }
         cm_GetServerNoLock(tsp);
@@ -864,7 +874,8 @@ long cm_ConnByMServers(cm_serverRef_t *serversp, cm_user_t *usersp,
 
     if (firstError == 0) {
         if (allDown) 
-            firstError = (reqp->tokenError ? reqp->tokenError : CM_ERROR_ALLDOWN);
+            firstError = (reqp->tokenError ? reqp->tokenError : 
+                          (reqp->idleError ? RX_CALL_TIMEOUT : CM_ERROR_ALLDOWN));
         else if (allBusy) 
             firstError = CM_ERROR_ALLBUSY;
 	else if (allOffline || (someBusy && someOffline))
@@ -948,6 +959,7 @@ static void cm_NewRXConnection(cm_conn_t *tcp, cm_ucell_t *ucellp,
                                   secIndex);
     rx_SetConnDeadTime(tcp->callp, ConnDeadtimeout);
     rx_SetConnHardDeadTime(tcp->callp, HardDeadtimeout);
+    rx_SetConnIdleDeadTime(tcp->callp, IdleDeadtimeout);
     tcp->ucgen = ucellp->gen;
     if (secObjp)
         rxs_Release(secObjp);   /* Decrement the initial refCount */
