@@ -27,6 +27,7 @@
 #include <rx/rx.h>
 
 osi_rwlock_t cm_serverLock;
+osi_rwlock_t cm_syscfgLock;
 
 cm_server_t *cm_allServersp;
 afs_uint32   cm_numFileServers = 0;
@@ -37,13 +38,13 @@ cm_ForceNewConnectionsAllServers(void)
 {
     cm_server_t *tsp;
 
-    lock_ObtainWrite(&cm_serverLock);
+    lock_ObtainRead(&cm_serverLock);
     for (tsp = cm_allServersp; tsp; tsp = tsp->allNextp) {
         cm_GetServerNoLock(tsp);
 	cm_ForceNewConnections(tsp);
         cm_PutServerNoLock(tsp);
     }
-    lock_ReleaseWrite(&cm_serverLock);
+    lock_ReleaseRead(&cm_serverLock);
 }
 
 void 
@@ -141,7 +142,7 @@ cm_PingServer(cm_server_t *tsp)
                         cm_InitReq(&req);
 
                         lock_ReleaseMutex(&tsp->mx);
-                        code = cm_GetVolumeByID(tsp->cellp, tsrvp->ids[i], cm_rootUserp,
+                        code = cm_FindVolumeByID(tsp->cellp, tsrvp->ids[i], cm_rootUserp,
                                                 &req, CM_GETVOL_FLAG_NO_LRU_UPDATE, &volp);
                         lock_ObtainMutex(&tsp->mx);
                         if (code == 0) {
@@ -178,7 +179,7 @@ cm_PingServer(cm_server_t *tsp)
                         cm_InitReq(&req);
 
                         lock_ReleaseMutex(&tsp->mx);
-                        code = cm_GetVolumeByID(tsp->cellp, tsrvp->ids[i], cm_rootUserp,
+                        code = cm_FindVolumeByID(tsp->cellp, tsrvp->ids[i], cm_rootUserp,
                                                 &req, CM_GETVOL_FLAG_NO_LRU_UPDATE, &volp);
                         lock_ObtainMutex(&tsp->mx);
                         if (code == 0) {
@@ -211,10 +212,10 @@ void cm_CheckServers(afs_uint32 flags, cm_cell_t *cellp)
     int isDown;
     int isFS;
 
-    lock_ObtainWrite(&cm_serverLock);
+    lock_ObtainRead(&cm_serverLock);
     for (tsp = cm_allServersp; tsp; tsp = tsp->allNextp) {
         cm_GetServerNoLock(tsp);
-        lock_ReleaseWrite(&cm_serverLock);
+        lock_ReleaseRead(&cm_serverLock);
 
         /* now process the server */
         lock_ObtainMutex(&tsp->mx);
@@ -249,10 +250,10 @@ void cm_CheckServers(afs_uint32 flags, cm_cell_t *cellp)
          */
         cm_GCConnections(tsp);
 
-        lock_ObtainWrite(&cm_serverLock);
+        lock_ObtainRead(&cm_serverLock);
         cm_PutServerNoLock(tsp);
     }
-    lock_ReleaseWrite(&cm_serverLock);
+    lock_ReleaseRead(&cm_serverLock);
 }       
 #else /* MULTI_CHECKSERVERS */
 void cm_CheckServers(afs_uint32 flags, cm_cell_t *cellp)
@@ -302,7 +303,7 @@ void cm_CheckServers(afs_uint32 flags, cm_cell_t *cellp)
     if ((flags & CM_FLAG_CHECKFILESERVERS) || 
         !(flags & (CM_FLAG_CHECKFILESERVERS|CM_FLAG_CHECKVLDBSERVERS)))
     {
-        lock_ObtainWrite(&cm_serverLock);
+        lock_ObtainRead(&cm_serverLock);
         nconns = 0;
         for (nconns=0, tsp = cm_allServersp; tsp; tsp = tsp->allNextp) {
             if (tsp->type != CM_SERVER_FILE || 
@@ -311,7 +312,7 @@ void cm_CheckServers(afs_uint32 flags, cm_cell_t *cellp)
                 continue;
 
             cm_GetServerNoLock(tsp);
-            lock_ReleaseWrite(&cm_serverLock);
+            lock_ReleaseRead(&cm_serverLock);
 
             lock_ObtainMutex(&tsp->mx);
             isDown = tsp->flags & CM_SERVERFLAG_DOWN;
@@ -320,7 +321,8 @@ void cm_CheckServers(afs_uint32 flags, cm_cell_t *cellp)
                 !((isDown && (flags & CM_FLAG_CHECKDOWNSERVERS)) ||
                    (!isDown && (flags & CM_FLAG_CHECKUPSERVERS)))) {
                 lock_ReleaseMutex(&tsp->mx);
-                lock_ObtainWrite(&cm_serverLock);
+                lock_ObtainRead(&cm_serverLock);
+                cm_PutServerNoLock(tsp);
                 continue;
             }
 
@@ -330,18 +332,18 @@ void cm_CheckServers(afs_uint32 flags, cm_cell_t *cellp)
             serversp[nconns] = tsp;
             code = cm_ConnByServer(tsp, cm_rootUserp, &conns[nconns]);
             if (code) {
-	            lock_ObtainWrite(&cm_serverLock);
+                lock_ObtainRead(&cm_serverLock);
                 cm_PutServerNoLock(tsp);
                 continue;
             }
-            lock_ObtainWrite(&cm_serverLock);
-			rxconns[nconns] = cm_GetRxConn(conns[nconns]);
+            lock_ObtainRead(&cm_serverLock);
+            rxconns[nconns] = cm_GetRxConn(conns[nconns]);
             if (conntimer[nconns] = (isDown ? 1 : 0))
                 rx_SetConnDeadTime(rxconns[nconns], 10);
 
             nconns++;
         }
-        lock_ReleaseWrite(&cm_serverLock);
+        lock_ReleaseRead(&cm_serverLock);
 
         if (nconns) {
             /* Perform the multi call */
@@ -403,7 +405,7 @@ void cm_CheckServers(afs_uint32 flags, cm_cell_t *cellp)
                                 cm_InitReq(&req);
 
                                 lock_ReleaseMutex(&tsp->mx);
-                                code = cm_GetVolumeByID(tsp->cellp, tsrvp->ids[i], cm_rootUserp,
+                                code = cm_FindVolumeByID(tsp->cellp, tsrvp->ids[i], cm_rootUserp,
                                                          &req, CM_GETVOL_FLAG_NO_LRU_UPDATE, &volp);
                                 lock_ObtainMutex(&tsp->mx);
                                 if (code == 0) {
@@ -441,7 +443,7 @@ void cm_CheckServers(afs_uint32 flags, cm_cell_t *cellp)
                                 cm_InitReq(&req);
 
                                 lock_ReleaseMutex(&tsp->mx);
-                                code = cm_GetVolumeByID(tsp->cellp, tsrvp->ids[i], cm_rootUserp,
+                                code = cm_FindVolumeByID(tsp->cellp, tsrvp->ids[i], cm_rootUserp,
                                                          &req, CM_GETVOL_FLAG_NO_LRU_UPDATE, &volp);
                                 lock_ObtainMutex(&tsp->mx);
                                 if (code == 0) {
@@ -532,7 +534,7 @@ void cm_CheckServers(afs_uint32 flags, cm_cell_t *cellp)
                                 cm_InitReq(&req);
 
                                 lock_ReleaseMutex(&tsp->mx);
-                                code = cm_GetVolumeByID(tsp->cellp, tsrvp->ids[i], cm_rootUserp,
+                                code = cm_FindVolumeByID(tsp->cellp, tsrvp->ids[i], cm_rootUserp,
                                                          &req, CM_GETVOL_FLAG_NO_LRU_UPDATE, &volp);
                                 lock_ObtainMutex(&tsp->mx);
                                 if (code == 0) {
@@ -570,7 +572,7 @@ void cm_CheckServers(afs_uint32 flags, cm_cell_t *cellp)
                                 cm_InitReq(&req);
 
                                 lock_ReleaseMutex(&tsp->mx);
-                                code = cm_GetVolumeByID(tsp->cellp, tsrvp->ids[i], cm_rootUserp,
+                                code = cm_FindVolumeByID(tsp->cellp, tsrvp->ids[i], cm_rootUserp,
                                                          &req, CM_GETVOL_FLAG_NO_LRU_UPDATE, &volp);
                                 lock_ObtainMutex(&tsp->mx);
                                 if (code == 0) {
@@ -597,7 +599,7 @@ void cm_CheckServers(afs_uint32 flags, cm_cell_t *cellp)
     if ((flags & CM_FLAG_CHECKVLDBSERVERS) || 
         !(flags & (CM_FLAG_CHECKFILESERVERS|CM_FLAG_CHECKVLDBSERVERS)))
     {
-        lock_ObtainWrite(&cm_serverLock);
+        lock_ObtainRead(&cm_serverLock);
         nconns = 0;
         for (nconns=0, tsp = cm_allServersp; tsp; tsp = tsp->allNextp) {
             if (tsp->type != CM_SERVER_VLDB ||
@@ -606,7 +608,7 @@ void cm_CheckServers(afs_uint32 flags, cm_cell_t *cellp)
                 continue;
 
             cm_GetServerNoLock(tsp);
-            lock_ReleaseWrite(&cm_serverLock);
+            lock_ReleaseRead(&cm_serverLock);
 
             lock_ObtainMutex(&tsp->mx);
             isDown = tsp->flags & CM_SERVERFLAG_DOWN;
@@ -615,7 +617,8 @@ void cm_CheckServers(afs_uint32 flags, cm_cell_t *cellp)
                 !((isDown && (flags & CM_FLAG_CHECKDOWNSERVERS)) ||
                    (!isDown && (flags & CM_FLAG_CHECKUPSERVERS)))) {
                 lock_ReleaseMutex(&tsp->mx);
-                lock_ObtainWrite(&cm_serverLock);
+                lock_ObtainRead(&cm_serverLock);
+                cm_PutServerNoLock(tsp);
                 continue;
             }
 
@@ -625,11 +628,11 @@ void cm_CheckServers(afs_uint32 flags, cm_cell_t *cellp)
             serversp[nconns] = tsp;
             code = cm_ConnByServer(tsp, cm_rootUserp, &conns[nconns]);
             if (code) {
-	            lock_ObtainWrite(&cm_serverLock);
+                lock_ObtainRead(&cm_serverLock);
                 cm_PutServerNoLock(tsp);
                 continue;
             }
-            lock_ObtainWrite(&cm_serverLock);
+            lock_ObtainRead(&cm_serverLock);
             rxconns[nconns] = cm_GetRxConn(conns[nconns]);
             conntimer[nconns] = (isDown ? 1 : 0);
             if (isDown)
@@ -637,7 +640,7 @@ void cm_CheckServers(afs_uint32 flags, cm_cell_t *cellp)
 
             nconns++;
         }
-        lock_ReleaseWrite(&cm_serverLock);
+        lock_ReleaseRead(&cm_serverLock);
 
         if (nconns) {
             /* Perform the multi call */
@@ -686,7 +689,7 @@ void cm_CheckServers(afs_uint32 flags, cm_cell_t *cellp)
                                 cm_InitReq(&req);
 
                                 lock_ReleaseMutex(&tsp->mx);
-                                code = cm_GetVolumeByID(tsp->cellp, tsrvp->ids[i], cm_rootUserp,
+                                code = cm_FindVolumeByID(tsp->cellp, tsrvp->ids[i], cm_rootUserp,
                                                          &req, CM_GETVOL_FLAG_NO_LRU_UPDATE, &volp);
                                 lock_ObtainMutex(&tsp->mx);
                                 if (code == 0) {
@@ -724,7 +727,7 @@ void cm_CheckServers(afs_uint32 flags, cm_cell_t *cellp)
                                 cm_InitReq(&req);
 
                                 lock_ReleaseMutex(&tsp->mx);
-                                code = cm_GetVolumeByID(tsp->cellp, tsrvp->ids[i], cm_rootUserp,
+                                code = cm_FindVolumeByID(tsp->cellp, tsrvp->ids[i], cm_rootUserp,
                                                          &req, CM_GETVOL_FLAG_NO_LRU_UPDATE, &volp);
                                 lock_ObtainMutex(&tsp->mx);
                                 if (code == 0) {
@@ -763,32 +766,51 @@ void cm_InitServer(void)
         
     if (osi_Once(&once)) {
         lock_InitializeRWLock(&cm_serverLock, "cm_serverLock");
+        lock_InitializeRWLock(&cm_syscfgLock, "cm_syscfgLock");
         osi_EndOnce(&once);
     }
 }
 
+/* Protected by cm_syscfgLock (rw) */
+int cm_noIPAddr;         /* number of client network interfaces */
+int cm_IPAddr[CM_MAXINTERFACE_ADDR];    /* client's IP address in host order */
+int cm_SubnetMask[CM_MAXINTERFACE_ADDR];/* client's subnet mask in host order*/
+int cm_NetMtu[CM_MAXINTERFACE_ADDR];    /* client's MTU sizes */
+int cm_NetFlags[CM_MAXINTERFACE_ADDR];  /* network flags */
+int cm_LanAdapterChangeDetected = 1;
+
+void cm_SetLanAdapterChangeDetected(void)
+{
+    lock_ObtainWrite(&cm_syscfgLock);
+    cm_LanAdapterChangeDetected = 1;
+    lock_ReleaseWrite(&cm_syscfgLock);
+}
+
 void cm_GetServer(cm_server_t *serverp)
 {
-    lock_ObtainWrite(&cm_serverLock);
-    serverp->refCount++;
-    lock_ReleaseWrite(&cm_serverLock);
+    lock_ObtainRead(&cm_serverLock);
+    InterlockedIncrement(&serverp->refCount);
+    lock_ReleaseRead(&cm_serverLock);
 }
 
 void cm_GetServerNoLock(cm_server_t *serverp)
 {
-    serverp->refCount++;
+    InterlockedIncrement(&serverp->refCount);
 }
 
 void cm_PutServer(cm_server_t *serverp)
 {
-    lock_ObtainWrite(&cm_serverLock);
-    osi_assertx(serverp->refCount-- > 0, "cm_server_t refCount 0");
-    lock_ReleaseWrite(&cm_serverLock);
+    afs_int32 refCount;
+    lock_ObtainRead(&cm_serverLock);
+    refCount = InterlockedDecrement(&serverp->refCount);
+    osi_assertx(refCount >= 0, "cm_server_t refCount underflow");
+    lock_ReleaseRead(&cm_serverLock);
 }
 
 void cm_PutServerNoLock(cm_server_t *serverp)
 {
-    osi_assertx(serverp->refCount-- > 0, "cm_server_t refCount 0");
+    afs_int32 refCount = InterlockedDecrement(&serverp->refCount);
+    osi_assertx(refCount >= 0, "cm_server_t refCount underflow");
 }
 
 void cm_SetServerNo64Bit(cm_server_t * serverp, int no64bit)
@@ -817,19 +839,23 @@ void cm_SetServerPrefs(cm_server_t * serverp)
     unsigned long	myAddr, myNet, mySubnet;/* in host byte order */
     unsigned long	netMask;
     int 		i;
-
-    int cm_noIPAddr;         /* number of client network interfaces */
-    int cm_IPAddr[CM_MAXINTERFACE_ADDR];    /* client's IP address in host order */
-    int cm_SubnetMask[CM_MAXINTERFACE_ADDR];/* client's subnet mask in host order*/
-    int cm_NetMtu[CM_MAXINTERFACE_ADDR];    /* client's MTU sizes */
-    int cm_NetFlags[CM_MAXINTERFACE_ADDR];  /* network flags */
     long code;
+    int writeLock = 0;
 
-    /* get network related info */
-    cm_noIPAddr = CM_MAXINTERFACE_ADDR;
-    code = syscfg_GetIFInfo(&cm_noIPAddr,
-			    cm_IPAddr, cm_SubnetMask,
-			    cm_NetMtu, cm_NetFlags);
+    lock_ObtainRead(&cm_syscfgLock);
+    if (cm_LanAdapterChangeDetected) {
+        lock_ConvertRToW(&cm_syscfgLock);
+        writeLock = 1;
+        if (cm_LanAdapterChangeDetected) {
+            /* get network related info */
+            cm_noIPAddr = CM_MAXINTERFACE_ADDR;
+            code = syscfg_GetIFInfo(&cm_noIPAddr,
+                                     cm_IPAddr, cm_SubnetMask,
+                                     cm_NetMtu, cm_NetFlags);
+            cm_LanAdapterChangeDetected = 0;
+        }
+        lock_ConvertWToR(&cm_syscfgLock);
+    }
 
     serverAddr = ntohl(serverp->addr.sin_addr.s_addr);
     serverp->ipRank  = CM_IPRANK_LOW;	/* default setings */
@@ -868,6 +894,7 @@ void cm_SetServerPrefs(cm_server_t * serverp)
 	/* random between 0..15*/
 	serverp->ipRank += min(serverp->ipRank, rand() % 0x000f);
     } /* and of for loop */
+    lock_ReleaseRead(&cm_syscfgLock);
 }
 
 cm_server_t *cm_NewServer(struct sockaddr_in *socketp, int type, cm_cell_t *cellp, afs_uint32 flags) {
@@ -932,7 +959,7 @@ cm_server_t *cm_FindServer(struct sockaddr_in *addrp, int type)
 
     osi_assertx(addrp->sin_family == AF_INET, "unexpected socket value");
         
-    lock_ObtainWrite(&cm_serverLock);
+    lock_ObtainRead(&cm_serverLock);
     for (tsp = cm_allServersp; tsp; tsp=tsp->allNextp) {
         if (tsp->type == type &&
             tsp->addr.sin_addr.s_addr == addrp->sin_addr.s_addr) 
@@ -944,7 +971,7 @@ cm_server_t *cm_FindServer(struct sockaddr_in *addrp, int type)
         cm_GetServerNoLock(tsp);
 
     /* drop big table lock */
-    lock_ReleaseWrite(&cm_serverLock);
+    lock_ReleaseRead(&cm_serverLock);
 	
     /* return what we found */
     return tsp;
@@ -1026,7 +1053,7 @@ LONG_PTR cm_ChecksumServerList(cm_serverRef_t *serversp)
     int first = 1;
     cm_serverRef_t *tsrp;
 
-    lock_ObtainWrite(&cm_serverLock);
+    lock_ObtainRead(&cm_serverLock);
     for (tsrp = serversp; tsrp; tsrp=tsrp->next) {
         if (first)
             first = 0;
@@ -1035,13 +1062,13 @@ LONG_PTR cm_ChecksumServerList(cm_serverRef_t *serversp)
         sum ^= (LONG_PTR) tsrp->server;
     }
 
-    lock_ReleaseWrite(&cm_serverLock);
+    lock_ReleaseRead(&cm_serverLock);
     return sum;
 }
 
 /*
 ** Insert a server into the server list keeping the list sorted in 
-** asending order of ipRank. 
+** ascending order of ipRank. 
 ** 
 ** The refCount of the cm_serverRef_t is increased
 */
