@@ -18,7 +18,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_open.c,v 1.10 2003/07/15 23:14:30 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_open.c,v 1.11.20.1 2008/05/20 21:59:40 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -123,11 +123,11 @@ afs_open(struct vcache **avcp, afs_int32 aflags, struct AFS_UCRED *acred)
 #endif
 	/* normal file or symlink */
 	osi_FlushText(tvc);	/* only needed to flush text if text locked last time */
-#if defined(AFS_SUN_ENV) || defined(AFS_ALPHA_ENV) || defined(AFS_SUN5_ENV)
+#ifdef AFS_BOZONLOCK_ENV
 	afs_BozonLock(&tvc->pvnLock, tvc);
 #endif
 	osi_FlushPages(tvc, acred);
-#if defined(AFS_SUN_ENV) || defined(AFS_ALPHA_ENV) || defined(AFS_SUN5_ENV)
+#ifdef AFS_BOZONLOCK_ENV
 	afs_BozonUnlock(&tvc->pvnLock, tvc);
 #endif
     }
@@ -150,6 +150,35 @@ afs_open(struct vcache **avcp, afs_int32 aflags, struct AFS_UCRED *acred)
     }
 #endif
     ReleaseReadLock(&tvc->lock);
+    if ((afs_preCache != 0) && (writing == 0) && (vType(tvc) != VDIR) && 
+	(!afs_BBusy())) {
+	register struct dcache *tdc;
+	afs_size_t offset, len, totallen = 0;
+
+	tdc = afs_GetDCache(tvc, 0, &treq, &offset, &len, 1);
+
+	ObtainSharedLock(&tdc->mflock, 865);
+	if (!(tdc->mflags & DFFetchReq)) {
+	    struct brequest *bp;
+
+	    /* start the daemon (may already be running, however) */
+	    UpgradeSToWLock(&tdc->mflock, 666);
+	    tdc->mflags |= DFFetchReq;  /* guaranteed to be cleared by BKG or 
+					   GetDCache */
+	    /* last parm (1) tells bkg daemon to do an afs_PutDCache when it 
+	       is done, since we don't want to wait for it to finish before 
+	       doing so ourselves.
+	    */
+	    bp = afs_BQueue(BOP_FETCH, tvc, B_DONTWAIT, 0, acred,
+			    (afs_size_t) 0, (afs_size_t) 1, tdc);
+	    if (!bp) {
+		tdc->mflags &= ~DFFetchReq;
+	    }
+	    ReleaseWriteLock(&tdc->mflock);
+	} else {
+	    ReleaseSharedLock(&tdc->mflock);
+	}
+    }	
   done:
     afs_PutFakeStat(&fakestate);
     code = afs_CheckCode(code, &treq, 4);	/* avoid AIX -O bug */
