@@ -11,19 +11,19 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/afs_memcache.c,v 1.15 2003/07/15 23:14:12 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/afs_memcache.c,v 1.22.4.1 2007/06/23 15:29:36 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #ifndef AFS_LINUX22_ENV
 #include "rpc/types.h"
 #endif
-#ifdef	AFS_ALPHA_ENV
+#ifdef	AFS_OSF_ENV
 #undef kmem_alloc
 #undef kmem_free
 #undef mem_alloc
 #undef mem_free
 #undef register
-#endif /* AFS_ALPHA_ENV */
+#endif /* AFS_OSF_ENV */
 #include "afsincludes.h"	/* Afs-based standard headers */
 #include "afs/afs_stats.h"	/* statistics */
 
@@ -84,13 +84,18 @@ afs_InitMemCache(int blkCount, int blkSize, int flags)
 }
 
 int
-afs_MemCacheClose(char *file)
+afs_MemCacheClose(struct osi_file *file)
 {
     return 0;
 }
 
+#if defined(AFS_SUN57_64BIT_ENV) || defined(AFS_SGI62_ENV)
 void *
 afs_MemCacheOpen(ino_t blkno)
+#else
+void *
+afs_MemCacheOpen(afs_int32 blkno)
+#endif
 {
     struct memCacheEntry *mep;
 
@@ -98,9 +103,8 @@ afs_MemCacheOpen(ino_t blkno)
 	osi_Panic("afs_MemCacheOpen: invalid block #");
     }
     mep = (memCache + blkno);
-    afs_Trace4(afs_iclSetp, CM_TRACE_MEMOPEN, ICL_TYPE_INT32, blkno,
-	       ICL_TYPE_POINTER, mep, ICL_TYPE_POINTER, mep->data,
-	       ICL_TYPE_STRING, mep->data);
+    afs_Trace3(afs_iclSetp, CM_TRACE_MEMOPEN, ICL_TYPE_INT32, blkno,
+	       ICL_TYPE_POINTER, mep, ICL_TYPE_POINTER, mep ? mep->data : 0);
     return (void *)mep;
 }
 
@@ -108,9 +112,10 @@ afs_MemCacheOpen(ino_t blkno)
  * this routine simulates a read in the Memory Cache 
  */
 int
-afs_MemReadBlk(register struct memCacheEntry *mceP, int offset, char *dest,
+afs_MemReadBlk(register struct osi_file *fP, int offset, void *dest,
 	       int size)
 {
+    register struct memCacheEntry *mceP = (struct memCacheEntry *)fP;
     int bytesRead;
 
     MObtainReadLock(&mceP->afs_memLock);
@@ -175,22 +180,23 @@ afs_MemReadUIO(ino_t blkno, struct uio *uioP)
 {
     register struct memCacheEntry *mceP =
 	(struct memCacheEntry *)afs_MemCacheOpen(blkno);
-    int length = mceP->size - uioP->uio_offset;
+    int length = mceP->size - AFS_UIO_OFFSET(uioP);
     afs_int32 code;
 
     AFS_STATCNT(afs_MemReadUIO);
     MObtainReadLock(&mceP->afs_memLock);
-    length = (length < uioP->uio_resid) ? length : uioP->uio_resid;
-    AFS_UIOMOVE(mceP->data + uioP->uio_offset, length, UIO_READ, uioP, code);
+    length = (length < AFS_UIO_RESID(uioP)) ? length : AFS_UIO_RESID(uioP);
+    AFS_UIOMOVE(mceP->data + AFS_UIO_OFFSET(uioP), length, UIO_READ, uioP, code);
     MReleaseReadLock(&mceP->afs_memLock);
     return code;
 }
 
 /*XXX: this extends a block arbitrarily to support big directories */
 int
-afs_MemWriteBlk(register struct memCacheEntry *mceP, int offset, char *src,
+afs_MemWriteBlk(register struct osi_file *fP, int offset, void *src,
 		int size)
 {
+    register struct memCacheEntry *mceP = (struct memCacheEntry *)fP;
     AFS_STATCNT(afs_MemWriteBlk);
     MObtainWriteLock(&mceP->afs_memLock, 560);
     if (size + offset > mceP->dataSize) {
@@ -241,6 +247,13 @@ afs_MemWritevBlk(register struct memCacheEntry *mceP, int offset,
 	char *oldData = mceP->data;
 
 	mceP->data = afs_osi_Alloc(size + offset);
+	if (mceP->data == NULL) {	/* no available memory */
+	    mceP->data = oldData;	/* revert back change that was made */
+	    MReleaseWriteLock(&mceP->afs_memLock);
+	    afs_warn("afs: afs_MemWriteBlk mem alloc failure (%d bytes)\n",
+		     size + offset);
+	    return -ENOMEM;
+	}
 
 	/* may overlap, but this is OK */
 	AFS_GUNLOCK();
@@ -275,41 +288,57 @@ afs_MemWriteUIO(ino_t blkno, struct uio *uioP)
 
     AFS_STATCNT(afs_MemWriteUIO);
     MObtainWriteLock(&mceP->afs_memLock, 312);
-    if (uioP->uio_resid + uioP->uio_offset > mceP->dataSize) {
+    if (AFS_UIO_RESID(uioP) + AFS_UIO_OFFSET(uioP) > mceP->dataSize) {
 	char *oldData = mceP->data;
 
-	mceP->data = afs_osi_Alloc(uioP->uio_resid + uioP->uio_offset);
+	mceP->data = afs_osi_Alloc(AFS_UIO_RESID(uioP) + AFS_UIO_OFFSET(uioP));
+	if (mceP->data == NULL) {	/* no available memory */
+	    mceP->data = oldData;	/* revert back change that was made */
+	    MReleaseWriteLock(&mceP->afs_memLock);
+	    afs_warn("afs: afs_MemWriteBlk mem alloc failure (%d bytes)\n",
+		     AFS_UIO_RESID(uioP) + AFS_UIO_OFFSET(uioP));
+	    return -ENOMEM;
+	}
 
 	AFS_GUNLOCK();
 	memcpy(mceP->data, oldData, mceP->size);
 	AFS_GLOCK();
 
 	afs_osi_Free(oldData, mceP->dataSize);
-	mceP->dataSize = uioP->uio_resid + uioP->uio_offset;
+	mceP->dataSize = AFS_UIO_RESID(uioP) + AFS_UIO_OFFSET(uioP);
     }
-    if (mceP->size < uioP->uio_offset)
+    if (mceP->size < AFS_UIO_OFFSET(uioP))
 	memset(mceP->data + mceP->size, 0,
-	       (int)(uioP->uio_offset - mceP->size));
-    AFS_UIOMOVE(mceP->data + uioP->uio_offset, uioP->uio_resid, UIO_WRITE,
+	       (int)(AFS_UIO_OFFSET(uioP) - mceP->size));
+    AFS_UIOMOVE(mceP->data + AFS_UIO_OFFSET(uioP), AFS_UIO_RESID(uioP), UIO_WRITE,
 		uioP, code);
-    if (uioP->uio_offset > mceP->size)
-	mceP->size = uioP->uio_offset;
+    if (AFS_UIO_OFFSET(uioP) > mceP->size)
+	mceP->size = AFS_UIO_OFFSET(uioP);
 
     MReleaseWriteLock(&mceP->afs_memLock);
     return code;
 }
 
 int
-afs_MemCacheTruncate(register struct memCacheEntry *mceP, int size)
+afs_MemCacheTruncate(register struct osi_file *fP, int size)
 {
+    register struct memCacheEntry *mceP = (struct memCacheEntry *)fP;
     AFS_STATCNT(afs_MemCacheTruncate);
 
     MObtainWriteLock(&mceP->afs_memLock, 313);
     /* old directory entry; g.c. */
     if (size == 0 && mceP->dataSize > memCacheBlkSize) {
-	afs_osi_Free(mceP->data, mceP->dataSize);
+	char *oldData = mceP->data;
 	mceP->data = afs_osi_Alloc(memCacheBlkSize);
-	mceP->dataSize = memCacheBlkSize;
+	if (mceP->data == NULL) {	/* no available memory */
+	    mceP->data = oldData;
+	    MReleaseWriteLock(&mceP->afs_memLock);
+	    afs_warn("afs: afs_MemWriteBlk mem alloc failure (%d bytes)\n",
+		     memCacheBlkSize);
+	} else {
+	    afs_osi_Free(oldData, mceP->dataSize);
+	    mceP->dataSize = memCacheBlkSize;
+	}
     }
 
     if (size < mceP->size)
@@ -321,11 +350,12 @@ afs_MemCacheTruncate(register struct memCacheEntry *mceP, int size)
 
 int
 afs_MemCacheStoreProc(register struct rx_call *acall,
-		      register struct memCacheEntry *mceP,
+		      register struct osi_file *fP,
 		      register afs_int32 alen, struct vcache *avc,
 		      int *shouldWake, afs_size_t * abytesToXferP,
-		      afs_size_t * abytesXferredP, afs_int32 length)
+		      afs_size_t * abytesXferredP)
 {
+    register struct memCacheEntry *mceP = (struct memCacheEntry *)fP;
 
     register afs_int32 code;
     register int tlen;
@@ -374,8 +404,9 @@ afs_MemCacheStoreProc(register struct rx_call *acall,
 	code = rx_WritevAlloc(acall, tiov, &tnio, RX_MAXIOVECS, tlen);
 	RX_AFS_GLOCK();
 	if (code <= 0) {
+	    code = rx_Error(acall);
 	    osi_FreeSmallSpace(tiov);
-	    return -33;
+	    return code ? code : -33;
 	}
 	tlen = code;
 	code = afs_MemReadvBlk(mceP, offset, tiov, tnio, tlen);
@@ -390,8 +421,9 @@ afs_MemCacheStoreProc(register struct rx_call *acall,
 	(*abytesXferredP) += code;
 #endif /* AFS_NOSTATS */
 	if (code != tlen) {
+	    code = rx_Error(acall);
 	    osi_FreeSmallSpace(tiov);
-	    return -33;
+	    return code ? code : -33;
 	}
 	offset += tlen;
 	alen -= tlen;
@@ -407,11 +439,12 @@ afs_MemCacheStoreProc(register struct rx_call *acall,
 
 int
 afs_MemCacheFetchProc(register struct rx_call *acall,
-		      register struct memCacheEntry *mceP, afs_size_t abase,
+		      register struct osi_file *fP, afs_size_t abase,
 		      struct dcache *adc, struct vcache *avc,
 		      afs_size_t * abytesToXferP, afs_size_t * abytesXferredP,
 		      afs_int32 lengthFound)
 {
+    register struct memCacheEntry *mceP = (struct memCacheEntry *)fP;
     register afs_int32 code;
     afs_int32 length;
     int moredata = 0;
