@@ -38,6 +38,8 @@
 #include <afsconfig.h>
 #include "afs/param.h"
 
+RCSID
+    ("$Header: /cvs/openafs/src/afs/afs_vcache.c,v 1.114.2.7.2.1 2008/05/20 19:52:29 shadow Exp $");
 
 #include "afs/sysincludes.h"	/*Standard vendor system headers */
 #include "afsincludes.h"	/*AFS-based standard headers */
@@ -75,7 +77,6 @@ struct afs_q afs_vhashTV[VCSIZE];
 static struct afs_cbr *afs_cbrHashT[CBRSIZE];
 afs_int32 afs_bulkStatsLost;
 int afs_norefpanic = 0;
-extern int afsd_dynamic_vcaches;
 
 /* Forward declarations */
 static afs_int32 afs_QueueVCB(struct vcache *avc);
@@ -208,7 +209,14 @@ afs_FlushVCache(struct vcache *avc, int *slept)
     vn_reinit(AFSTOV(avc));
 #endif
     afs_FreeAllAxs(&(avc->Access));
-    afs_QueueVCB(avc);
+
+    /* we can't really give back callbacks on RO files, since the
+     * server only tracks them on a per-volume basis, and we don't
+     * know whether we still have some other files from the same
+     * volume. */
+    if ((avc->states & CRO) == 0 && avc->callback) {
+	afs_QueueVCB(avc);
+    }
     ObtainWriteLock(&afs_xcbhash, 460);
     afs_DequeueCallback(avc);	/* remove it from queued callbacks list */
     avc->states &= ~(CStatd | CUnique);
@@ -245,7 +253,6 @@ afs_FlushVCache(struct vcache *avc, int *slept)
 	VN_UNLOCK(AFSTOV(avc));
 #endif
 	AFS_RELE(AFSTOV(avc));
-	afs_stats_cmperf.vcacheXAllocs--;
     } else {
 	if (afs_norefpanic) {
 	    printf("flush vc refcnt < 1");
@@ -306,8 +313,6 @@ afs_InactiveVCache(struct vcache *avc, struct AFS_UCRED *acred)
  * held, so we don't have to worry about blocking in osi_Alloc.
  */
 static struct afs_cbr *afs_cbrSpace = 0;
-/* if alloc limit below changes, fix me! */
-static struct afs_cbr *afs_cbrHeads[2];
 struct afs_cbr *
 afs_AllocCBR(void)
 {
@@ -329,7 +334,6 @@ afs_AllocCBR(void)
 	    }
 	    tsp[AFS_NCBRS - 1].next = 0;
 	    afs_cbrSpace = tsp;
-	    afs_cbrHeads[afs_stats_cmperf.CallBackAlloced] = tsp;
 	    afs_stats_cmperf.CallBackAlloced++;
 	}
     }
@@ -387,7 +391,7 @@ afs_FlushVCBs(afs_int32 lockit)
     struct server *tsp;
     int i;
     struct vrequest treq;
-    struct afs_conn *tc;
+    struct conn *tc;
     int safety1, safety2, safety3;
     XSTATS_DECLS;
     if ((code = afs_InitReq(&treq, afs_osi_credp)))
@@ -498,36 +502,22 @@ afs_FlushVCBs(afs_int32 lockit)
  * Environment:
  *	Locks the xvcb lock.
  *	Called when the xvcache lock is already held.
- *
- * \param avc vcache entry
- * \return 1 if queued, 0 otherwise
  */
 
 static afs_int32
 afs_QueueVCB(struct vcache *avc)
 {
-    int queued = 0;
     struct server *tsp;
     struct afs_cbr *tcbp;
 
     AFS_STATCNT(afs_QueueVCB);
-
-    MObtainWriteLock(&afs_xvcb, 274);
-
-    /* we can't really give back callbacks on RO files, since the
-     * server only tracks them on a per-volume basis, and we don't
-     * know whether we still have some other files from the same
-     * volume. */
-    if (!((avc->states & CRO) == 0 && avc->callback)) {
-        goto done;
-    }
-
     /* The callback is really just a struct server ptr. */
     tsp = (struct server *)(avc->callback);
 
     /* we now have a pointer to the server, so we just allocate
      * a queue entry and queue it.
      */
+    MObtainWriteLock(&afs_xvcb, 274);
     tcbp = afs_AllocCBR();
     tcbp->fid = avc->fid.Fid;
 
@@ -539,12 +529,10 @@ afs_QueueVCB(struct vcache *avc)
     tcbp->pprev = &tsp->cbrs;
 
     afs_InsertHashCBR(tcbp);
-    queued = 1;
 
- done:
     /* now release locks and return */
     MReleaseWriteLock(&afs_xvcb);
-    return queued;
+    return 0;
 }
 
 
@@ -611,7 +599,7 @@ afs_FlushReclaimedVcaches(void)
 	       We probably need a way to be smarter about this. */
 	    tvc->nextfree = tmpReclaimedVCList;
 	    tmpReclaimedVCList = tvc;
-	    printf("Reclaim list flush %x failed: %d\n", tvc, code);
+	    printf("Reclaim list flush %lx failed: %d\n", (unsigned long) tvc, code);
 	}
         if (tvc->states & (CVInit
 #ifdef AFS_DARWIN80_ENV
@@ -633,22 +621,54 @@ afs_FlushReclaimedVcaches(void)
 #endif
 }
 
-int
-afs_ShakeLooseVCaches(afs_int32 anumber)
+/*
+ * afs_NewVCache
+ *
+ * Description:
+ *	This routine is responsible for allocating a new cache entry
+ *	from the free list.  It formats the cache entry and inserts it
+ *	into the appropriate hash tables.  It must be called with
+ *	afs_xvcache write-locked so as to prevent several processes from
+ *	trying to create a new cache entry simultaneously.
+ *
+ * Parameters:
+ *	afid  : The file id of the file whose cache entry is being
+ *		created.
+ */
+/* LOCK: afs_NewVCache  afs_xvcache W */
+struct vcache *
+afs_NewVCache(struct VenusFid *afid, struct server *serverp)
 {
-#if defined(AFS_OSF_ENV) || defined(AFS_LINUX22_ENV)
-    afs_int32 i, j;
     struct vcache *tvc;
+    afs_int32 i, j;
+    afs_int32 anumber = VCACHE_FREE;
+#ifdef	AFS_AIX_ENV
+    struct gnode *gnodepnt;
+#endif
+#ifdef	AFS_OSF_ENV
+    struct vcache *nvc;
+#endif /* AFS_OSF_ENV */
     struct afs_q *tq, *uq;
     int code, fv_slept;
-    afs_int32 target = anumber;
 
-    if (
-#ifdef AFS_MAXVCOUNT_ENV
-	afsd_dynamic_vcaches || /* Always run if dynamic vcaches are enabled. */
+    AFS_STATCNT(afs_NewVCache);
+
+    afs_FlushReclaimedVcaches();
+
+#if defined(AFS_OSF_ENV) || defined(AFS_LINUX22_ENV)
+#if defined(AFS_OSF30_ENV) || defined(AFS_LINUX22_ENV)
+    if (afs_vcount >= afs_maxvcount)
+#else
+    /*
+     * If we are using > 33 % of the total system vnodes for AFS vcache
+     * entries or we are using the maximum number of vcache entries,
+     * then free some.  (if our usage is > 33% we should free some, if
+     * our usage is > afs_maxvcount, set elsewhere to 0.5*nvnode,
+     * we _must_ free some -- no choice).
+     */
+    if (((3 * afs_vcount) > nvnode) || (afs_vcount >= afs_maxvcount))
 #endif
-	afs_vcount >= afs_maxvcount
-	) {
+    {
 	int i;
 	char *panicstr;
 
@@ -658,11 +678,7 @@ afs_ShakeLooseVCaches(afs_int32 anumber)
 	    uq = QPrev(tq);
 	    if (tvc->states & CVFlushed) {
 		refpanic("CVFlushed on VLRU");
-	    } else if (
-#ifdef AFS_MAXVCOUNT_ENV
-	    ! afsd_dynamic_vcaches && 
-#endif
-	    i++ > afs_maxvcount) {
+	    } else if (i++ > afs_maxvcount) {
 		refpanic("Exceeded pool of AFS vnodes(VLRU cycle?)");
 	    } else if (QNext(uq) != tq) {
 		refpanic("VLRU inconsistent");
@@ -673,15 +689,12 @@ afs_ShakeLooseVCaches(afs_int32 anumber)
 #if defined(AFS_LINUX22_ENV)
 	    if (tvc != afs_globalVp && VREFCOUNT(tvc) > 1 && tvc->opens == 0) {
                 struct dentry *dentry;
-		struct inode *inode = AFSTOV(tvc);
                 struct list_head *cur, *head;
                 AFS_GUNLOCK();
-
-#if defined(HAVE_DCACHE_LOCK)
 #if defined(AFS_LINUX24_ENV)
                 spin_lock(&dcache_lock);
 #endif
-		head = &inode->i_dentry;
+		head = &(AFSTOV(tvc))->i_dentry;
 
 restart:
                 cur = head;
@@ -690,6 +703,7 @@ restart:
 
 		    if (d_unhashed(dentry))
 			continue;
+
 		    dget_locked(dentry);
 
 #if defined(AFS_LINUX24_ENV)
@@ -709,35 +723,6 @@ restart:
 #if defined(AFS_LINUX24_ENV)
 		spin_unlock(&dcache_lock);
 #endif
-#else /* HAVE_DCACHE_LOCK */
-		spin_lock(&inode->i_lock);
-		head = &inode->i_dentry;
-
-restart:
-		cur = head;
-		while ((cur = cur->next) != head) {
-		    dentry = list_entry(cur, struct dentry, d_alias);
-
-		    spin_lock(&dentry->d_lock);
-		    if (d_unhashed(dentry)) {
-			spin_unlock(&dentry->d_lock);
-			continue;
-		    }
-		    spin_unlock(&dentry->d_lock);
-		    dget(dentry);
-
-		    spin_unlock(&inode->i_lock);
-		    if (d_invalidate(dentry) == -EBUSY) {
-			dput(dentry);
-			/* perhaps lock and try to continue? (use cur as head?) */
-			goto inuse;
-		    }
-		    dput(dentry);
-		    spin_lock(&inode->i_lock);
-		    goto restart;
-		}
-		spin_unlock(&inode->i_lock);
-#endif /* HAVE_DCACHE_LOCK */
 	    inuse:
 		AFS_GLOCK();
 	    }
@@ -759,46 +744,24 @@ restart:
 	    if (tq == uq)
 		break;
 	}
-	if (
-#ifdef AFS_MAXVCOUNT_ENV
-        !afsd_dynamic_vcaches &&
-#endif
-        anumber == target) {
-	    printf("afs_ShakeLooseVCaches: warning none freed, using %d of %d\n",
+	if (anumber == VCACHE_FREE) {
+	    printf("afs_NewVCache: warning none freed, using %d of %d\n",
 		   afs_vcount, afs_maxvcount);
+	    if (afs_vcount >= afs_maxvcount) {
+	    	printf("afs_NewVCache - none freed\n");
+		return NULL;
+	    }
 	}
     }
-/*
-    printf("recycled %d entries\n", target-anumber);
-*/
-#endif
-    return 0;
-}
 
-
-static struct vcache *
-afs_AllocVCache(void) 
+#if defined(AFS_LINUX22_ENV)
 {
-    struct vcache *tvc;
-#if defined(AFS_OSF30_ENV)
-    struct vcache *nvc;
-    AFS_GUNLOCK();
-    if (getnewvnode(MOUNT_AFS, &Afs_vnodeops, &nvc)) {
-	/* What should we do ???? */
-	osi_Panic("afs_AllocVCache: no more vnodes");
-    }
-    AFS_GLOCK();
-
-    tvc = nvc;
-    tvc->nextfree = NULL;
-    afs_vcount++;
-#elif defined(AFS_LINUX22_ENV)
     struct inode *ip;
 
     AFS_GUNLOCK();
     ip = new_inode(afs_globalVFS);
     if (!ip)
-	osi_Panic("afs_AllocVCache: no more inodes");
+	osi_Panic("afs_NewVCache: no more inodes");
     AFS_GLOCK();
 #if defined(STRUCT_SUPER_HAS_ALLOC_INODE)
     tvc = VTOAFS(ip);
@@ -807,90 +770,19 @@ afs_AllocVCache(void)
     ip->u.generic_ip = tvc;
     tvc->v = ip;
 #endif
-
-    afs_vcount++;
-#ifdef AFS_MAXVCOUNT_ENV
-    /* track the peak */
-    if (afsd_dynamic_vcaches && afs_maxvcount < afs_vcount) {
-	afs_maxvcount = afs_vcount;
-	/*printf("peak vnodes: %d\n", afs_maxvcount);*/
-    }
-#endif
-    afs_stats_cmperf.vcacheXAllocs++;	/* count in case we have a leak */
-#else
-    /* none free, making one is better than a panic */
-    afs_stats_cmperf.vcacheXAllocs++;	/* count in case we have a leak */
-    tvc = (struct vcache *)afs_osi_Alloc(sizeof(struct vcache));
-#if defined(AFS_DARWIN_ENV) && !defined(UKERNEL)
-    tvc->v = NULL; /* important to clean this, or use memset 0 */
-#endif
-#ifdef	KERNEL_HAVE_PIN
-    pin((char *)tvc, sizeof(struct vcache));	/* XXX */
-#endif
-#if defined(AFS_SGI_ENV)
-    {
-	char name[METER_NAMSZ];
-	memset(tvc, 0, sizeof(struct vcache));
-	tvc->v.v_number = ++afsvnumbers;
-	tvc->vc_rwlockid = OSI_NO_LOCKID;
-	initnsema(&tvc->vc_rwlock, 1,
-		  makesname(name, "vrw", tvc->v.v_number));
-#ifndef	AFS_SGI53_ENV
-	initnsema(&tvc->v.v_sync, 0,
-		  makesname(name, "vsy", tvc->v.v_number));
-#endif
-#ifndef AFS_SGI62_ENV
-	initnlock(&tvc->v.v_lock,
-		  makesname(name, "vlk", tvc->v.v_number));
-#endif
-    }
-#endif /* AFS_SGI_ENV */
-#endif
-    return tvc;
 }
-
-/*!
- *   This routine is responsible for allocating a new cache entry
- * from the free list.  It formats the cache entry and inserts it
- * into the appropriate hash tables.  It must be called with
- * afs_xvcache write-locked so as to prevent several processes from
- * trying to create a new cache entry simultaneously.
- *
- * LOCK: afs_NewVCache  afs_xvcache W
- *
- * \param afid The file id of the file whose cache entry is being created.
- *
- * \return The new vcache struct.
- */
-struct vcache *
-afs_NewVCache(struct VenusFid *afid, struct server *serverp)
-{
-    struct vcache *tvc;
-    afs_int32 i, j;
-    afs_int32 anumber = VCACHE_FREE;
-#ifdef	AFS_AIX_ENV
-    struct gnode *gnodepnt;
-#endif
-    struct afs_q *tq, *uq;
-    int code, fv_slept;
-
-    AFS_STATCNT(afs_NewVCache);
-
-    afs_FlushReclaimedVcaches();
-
-#if defined(AFS_OSF_ENV) || defined(AFS_LINUX22_ENV)
-#ifdef AFS_MAXVCOUNT_ENV
-    if(!afsd_dynamic_vcaches) {
-#endif
-    afs_ShakeLooseVCaches(anumber);
-    if (afs_vcount >= afs_maxvcount) {
-	printf("afs_NewVCache - none freed\n");
-	return NULL;
-    }
-#ifdef AFS_MAXVCOUNT_ENV
+#else
+    AFS_GUNLOCK();
+    if (getnewvnode(MOUNT_AFS, &Afs_vnodeops, &nvc)) {
+	/* What should we do ???? */
+	osi_Panic("afs_NewVCache: no more vnodes");
     }
+    AFS_GLOCK();
+
+    tvc = nvc;
+    tvc->nextfree = NULL;
 #endif
-    tvc = afs_AllocVCache();
+    afs_vcount++;
 #else /* AFS_OSF_ENV */
     /* pull out a free cache entry */
     if (!freeVCList) {
@@ -978,7 +870,33 @@ afs_NewVCache(struct VenusFid *afid, struct server *serverp)
 	}
     }
     if (!freeVCList) {
-	tvc = afs_AllocVCache();
+	/* none free, making one is better than a panic */
+	afs_stats_cmperf.vcacheXAllocs++;	/* count in case we have a leak */
+	tvc = (struct vcache *)afs_osi_Alloc(sizeof(struct vcache));
+#if defined(AFS_DARWIN_ENV) && !defined(UKERNEL)
+	tvc->v = NULL; /* important to clean this, or use memset 0 */
+#endif
+#ifdef	KERNEL_HAVE_PIN
+	pin((char *)tvc, sizeof(struct vcache));	/* XXX */
+#endif
+#if defined(AFS_SGI_ENV)
+	{
+	    char name[METER_NAMSZ];
+	    memset(tvc, 0, sizeof(struct vcache));
+	    tvc->v.v_number = ++afsvnumbers;
+	    tvc->vc_rwlockid = OSI_NO_LOCKID;
+	    initnsema(&tvc->vc_rwlock, 1,
+		      makesname(name, "vrw", tvc->v.v_number));
+#ifndef	AFS_SGI53_ENV
+	    initnsema(&tvc->v.v_sync, 0,
+		      makesname(name, "vsy", tvc->v.v_number));
+#endif
+#ifndef AFS_SGI62_ENV
+	    initnlock(&tvc->v.v_lock,
+		      makesname(name, "vlk", tvc->v.v_number));
+#endif
+	}
+#endif /* AFS_SGI_ENV */
     } else {
 	tvc = freeVCList;	/* take from free list */
 	freeVCList = tvc->nextfree;
@@ -1204,11 +1122,6 @@ afs_NewVCache(struct VenusFid *afid, struct server *serverp)
     insmntque(tvc, afs_globalVFS);
 #endif /* AFS_OSF_ENV */
 #endif /* AFS_DUX40_ENV */
-#ifdef AFS_FBSD70_ENV
-#ifndef AFS_FBSD80_ENV /* yup.  they put it back. */
-    insmntque(AFSTOV(tvc), afs_globalVFS);
-#endif
-#endif
 #if defined(AFS_SGI_ENV)
     VN_SET_DPAGES(&(tvc->v), (struct pfdat *)NULL);
     osi_Assert((tvc->v.v_flag & VINACT) == 0);
@@ -1255,9 +1168,9 @@ afs_FlushActiveVcaches(register afs_int32 doflocks)
 {
     register struct vcache *tvc;
     register int i;
-    register struct afs_conn *tc;
+    register struct conn *tc;
     register afs_int32 code;
-    struct AFS_UCRED *cred = NULL;
+    register struct AFS_UCRED *cred = NULL;
     struct vrequest treq, ureq;
     struct AFSVolSync tsync;
     int didCore;
@@ -1582,7 +1495,7 @@ afs_WriteVCache(register struct vcache *avc,
 		struct vrequest *areq)
 {
     afs_int32 code;
-    struct afs_conn *tc;
+    struct conn *tc;
     struct AFSFetchStatus OutStatus;
     struct AFSVolSync tsync;
     XSTATS_DECLS;
@@ -1741,7 +1654,7 @@ afs_RemoteLookup(register struct VenusFid *afid, struct vrequest *areq,
 {
     afs_int32 code;
     afs_uint32 start;
-    register struct afs_conn *tc;
+    register struct conn *tc;
     struct AFSFetchStatus OutDirStatus;
     XSTATS_DECLS;
     if (!name)
@@ -1839,7 +1752,8 @@ afs_GetVCache(register struct VenusFid *afid, struct vrequest *areq,
 	if (cached)
 	    *cached = 1;
 	osi_Assert((tvc->states & CVInit) == 0);
-	if (tvc->states & CStatd) {
+	/* If we are in readdir, return the vnode even if not statd */
+	if ((tvc->states & CStatd) || afs_InReadDir(tvc)) {
 	    ReleaseSharedLock(&afs_xvcache);
 	    return tvc;
 	}
@@ -1851,7 +1765,7 @@ afs_GetVCache(register struct VenusFid *afid, struct vrequest *areq,
 	newvcache = 1;
 
 	ConvertWToSLock(&afs_xvcache);
-	if (tvc == NULL)
+	if (!tvc)
 	{
 		ReleaseSharedLock(&afs_xvcache);
 		return NULL;
@@ -1908,20 +1822,6 @@ afs_GetVCache(register struct VenusFid *afid, struct vrequest *areq,
 	  ObtainWriteLock(&tvc->lock, 954);
 	if (!iheldthelock)
 	    VOP_UNLOCK(vp, LK_EXCLUSIVE, current_proc());
-#elif defined(AFS_FBSD80_ENV)
-	iheldthelock = VOP_ISLOCKED(vp);
-	if (!iheldthelock) {
-	    /* nosleep/sleep lock order reversal */
-	    int glocked = ISAFS_GLOCK();
-	    if (glocked)
-		AFS_GUNLOCK();
-	    vn_lock(vp, LK_EXCLUSIVE | LK_RETRY);
-	    if (glocked)
-		AFS_GLOCK();
-	}
-	vinvalbuf(vp, V_SAVE, curthread, PINOD, 0);
-	if (!iheldthelock)
-	    VOP_UNLOCK(vp, 0);
 #elif defined(AFS_FBSD60_ENV)
 	iheldthelock = VOP_ISLOCKED(vp, curthread);
 	if (!iheldthelock)
@@ -1995,9 +1895,24 @@ afs_GetVCache(register struct VenusFid *afid, struct vrequest *areq,
 	if (afs_DynrootNewVnode(tvc, &OutStatus)) {
 	    afs_ProcessFS(tvc, &OutStatus, areq);
 	    tvc->states |= CStatd | CUnique;
+	    tvc->parentVnode  = OutStatus.ParentVnode;
+	    tvc->parentUnique = OutStatus.ParentUnique;
 	    code = 0;
 	} else {
 	    code = afs_FetchStatus(tvc, afid, areq, &OutStatus);
+	    /* For the NFS translator's benefit, make sure
+	     * non-directory vnodes always have their parent FID set
+	     * correctly, even when created as a result of decoding an
+	     * NFS filehandle.  It would be nice to also do this for
+	     * directories, but we can't because the fileserver fills
+	     * in the FID of the directory itself instead of that of
+	     * its parent.
+	     */
+            if (!code && OutStatus.FileType != Directory &&
+		!tvc->parentVnode) {
+		tvc->parentVnode  = OutStatus.ParentVnode;
+		tvc->parentUnique = OutStatus.ParentUnique;
+            }
 	}
     }
 
@@ -2441,7 +2356,7 @@ afs_FetchStatus(struct vcache * avc, struct VenusFid * afid,
 {
     int code;
     afs_uint32 start = 0;
-    register struct afs_conn *tc;
+    register struct conn *tc;
     struct AFSCallBack CallBack;
     struct AFSVolSync tsync;
     struct volume *volp;
@@ -2538,7 +2453,7 @@ afs_FetchStatus(struct vcache * avc, struct VenusFid * afid,
 void
 afs_StuffVcache(register struct VenusFid *afid,
 		struct AFSFetchStatus *OutStatus,
-		struct AFSCallBack *CallBack, register struct afs_conn *tc,
+		struct AFSCallBack *CallBack, register struct conn *tc,
 		struct vrequest *areq)
 {
     register afs_int32 code, i, newvcache = 0;
@@ -2899,7 +2814,6 @@ afs_NFSFindVCache(struct vcache **avcp, struct VenusFid *afid)
 	    && ((tvc->fid.Fid.Unique & 0xffffff) == afid->Fid.Unique)
 	    && (tvc->fid.Cell == afid->Cell)) {
 	    if (tvc->states & CVInit) {
-		int lock;
 		ReleaseSharedLock(&afs_xvcache);
 		afs_osi_Sleep(&tvc->states);
 		goto loop;
@@ -3104,7 +3018,7 @@ shutdown_vcache(void)
      */
 
     {
-	register struct afs_q *tq, *uq = NULL;
+	register struct afs_q *tq, *uq;
 	register struct vcache *tvc;
 	for (tq = VLRU.prev; tq != &VLRU; tq = uq) {
 	    tvc = QTOV(tq);
@@ -3159,8 +3073,7 @@ shutdown_vcache(void)
 		    tvc->linkData = 0;
 		}
 
-		if (tvc->Access)
-		    afs_FreeAllAxs(&(tvc->Access));
+		afs_FreeAllAxs(&(tvc->Access));
 	    }
 	    afs_vhashT[i] = 0;
 	}
@@ -3168,9 +3081,8 @@ shutdown_vcache(void)
     /*
      * Free any leftover callback queue
      */
-    for (i = 0; i < afs_stats_cmperf.CallBackAlloced; i++) {
-	tsp = afs_cbrHeads[i];
-	afs_cbrHeads[i] = 0;
+    for (tsp = afs_cbrSpace; tsp; tsp = nsp) {
+	nsp = tsp->next;
 	afs_osi_Free((char *)tsp, AFS_NCBRS * sizeof(struct afs_cbr));
     }
     afs_cbrSpace = 0;
