@@ -16,7 +16,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/rx/LINUX/rx_knet.c,v 1.19 2003/07/15 23:16:22 shadow Exp $");
+    ("$Header: /cvs/openafs/src/rx/LINUX/rx_knet.c,v 1.32.2.6.2.1 2008/05/20 21:59:44 shadow Exp $");
 
 #include <linux/version.h>
 #ifdef AFS_LINUX22_ENV
@@ -25,25 +25,42 @@ RCSID
 #include "h/smp_lock.h"
 #endif
 #include <asm/uaccess.h>
+#ifdef ADAPT_PMTU
+#include <linux/errqueue.h>
+#include <linux/icmp.h>
+#endif
 
 /* rxk_NewSocket
  * open and bind RX socket
  */
-struct osi_socket *
-rxk_NewSocket(short aport)
+osi_socket *
+rxk_NewSocketHost(afs_uint32 ahost, short aport)
 {
     struct socket *sockp;
     struct sockaddr_in myaddr;
     int code;
+    KERNEL_SPACE_DECL;
+#ifdef ADAPT_PMTU
+    int pmtu = IP_PMTUDISC_WANT;
+    int do_recverr = 1;
+#else
+    int pmtu = IP_PMTUDISC_DONT;
+#endif
 
-
+    /* We need a better test for this. if you need it back, tell us
+     * how to detect it. 
+     */
+#ifdef LINUX_KERNEL_SOCK_CREATE_V
+    code = sock_create(AF_INET, SOCK_DGRAM, IPPROTO_UDP, &sockp, 0);
+#else
     code = sock_create(AF_INET, SOCK_DGRAM, IPPROTO_UDP, &sockp);
+#endif
     if (code < 0)
 	return NULL;
 
     /* Bind socket */
     myaddr.sin_family = AF_INET;
-    myaddr.sin_addr.s_addr = htonl(INADDR_ANY);
+    myaddr.sin_addr.s_addr = ahost;
     myaddr.sin_port = aport;
     code =
 	sockp->ops->bind(sockp, (struct sockaddr *)&myaddr, sizeof(myaddr));
@@ -57,9 +74,22 @@ rxk_NewSocket(short aport)
 	return NULL;
     }
 
-    return (struct osi_socket *)sockp;
+    TO_USER_SPACE();
+    sockp->ops->setsockopt(sockp, SOL_IP, IP_MTU_DISCOVER, (char *)&pmtu,
+                           sizeof(pmtu));
+#ifdef ADAPT_PMTU
+    sockp->ops->setsockopt(sockp, SOL_IP, IP_RECVERR, (char *)&do_recverr,
+                           sizeof(do_recverr));
+#endif
+    TO_KERNEL_SPACE();
+    return (osi_socket *)sockp;
 }
 
+osi_socket *
+rxk_NewSocket(short aport)
+{
+    return rxk_NewSocketHost(htonl(INADDR_ANY), aport);
+}
 
 /* free socket allocated by osi_NetSocket */
 int
@@ -69,6 +99,65 @@ rxk_FreeSocket(register struct socket *asocket)
     return 0;
 }
 
+#ifdef ADAPT_PMTU
+void
+handle_socket_error(osi_socket so)
+{
+    KERNEL_SPACE_DECL;
+    struct msghdr msg;
+    struct cmsghdr *cmsg;
+    struct sock_extended_err *err;
+    struct sockaddr_in addr;
+    struct sockaddr *offender;
+    char *controlmsgbuf;
+    int code;
+    struct socket *sop = (struct socket *)so;
+
+    if (!(controlmsgbuf=rxi_Alloc(256)))
+	return;
+    msg.msg_name = &addr;
+    msg.msg_namelen = sizeof(addr);
+    msg.msg_iov = NULL;
+    msg.msg_iovlen = 0;
+    msg.msg_control = controlmsgbuf;
+    msg.msg_controllen = 256;
+    msg.msg_flags = 0;
+
+    TO_USER_SPACE();
+    code = sock_recvmsg(sop, &msg, 256, MSG_ERRQUEUE|MSG_DONTWAIT|MSG_TRUNC);
+    TO_KERNEL_SPACE();
+
+    if (code < 0 || !(msg.msg_flags & MSG_ERRQUEUE))
+	goto out;
+
+    for (cmsg = CMSG_FIRSTHDR(&msg); cmsg; cmsg = CMSG_NXTHDR(&msg, cmsg)) {
+	if (CMSG_OK(&msg, cmsg) && cmsg->cmsg_level == SOL_IP &&
+	    cmsg->cmsg_type == IP_RECVERR)
+	    break;
+    }
+    if (!cmsg)
+	goto out;
+    err = CMSG_DATA(cmsg);
+    offender = SO_EE_OFFENDER(err);
+    
+    if (offender->sa_family != AF_INET)
+       goto out;
+
+    memcpy(&addr, offender, sizeof(addr));
+
+    if (err->ee_origin == SO_EE_ORIGIN_ICMP &&
+	err->ee_type == ICMP_DEST_UNREACH &&
+	err->ee_code == ICMP_FRAG_NEEDED) {
+	rxi_SetPeerMtu(ntohl(addr.sin_addr.s_addr), ntohs(addr.sin_port),
+		       err->ee_info);
+    }
+    /* other DEST_UNREACH's and TIME_EXCEEDED should be dealt with too */
+
+out:
+    rxi_Free(controlmsgbuf, 256);
+    return;
+}
+#endif
 
 /* osi_NetSend
  *
@@ -77,27 +166,30 @@ rxk_FreeSocket(register struct socket *asocket)
  * non-zero = failure
  */
 int
-osi_NetSend(osi_socket sop, struct sockaddr_in *to, struct iovec *iov,
+osi_NetSend(osi_socket sop, struct sockaddr_in *to, struct iovec *iovec,
 	    int iovcnt, afs_int32 size, int istack)
 {
     KERNEL_SPACE_DECL;
     struct msghdr msg;
-    int code;
-    struct iovec tmpvec[RX_MAXWVECS + 2];
+    int code, sockerr;
+    size_t esize;
 
-    if (iovcnt > RX_MAXWVECS + 2) {
-	osi_Panic("Too many (%d) iovecs passed to osi_NetSend\n", iovcnt);
+#ifdef ADAPT_PMTU
+    while (1) {
+	sockerr=0;
+	esize = sizeof(sockerr);
+	TO_USER_SPACE();
+	sop->ops->getsockopt(sop, SOL_SOCKET, SO_ERROR, (char *)&sockerr,
+			   &esize);
+	TO_KERNEL_SPACE();
+	if (sockerr == 0)
+	   break;
+	handle_socket_error(sop);
     }
+#endif
 
-    if (iovcnt <= 2) {		/* avoid pointless uiomove */
-	tmpvec[0].iov_base = iov[0].iov_base;
-	tmpvec[0].iov_len = size;
-	msg.msg_iovlen = 1;
-    } else {
-	memcpy(tmpvec, iov, iovcnt * sizeof(struct iovec));
-	msg.msg_iovlen = iovcnt;
-    }
-    msg.msg_iov = tmpvec;
+    msg.msg_iovlen = iovcnt;
+    msg.msg_iov = iovec;
     msg.msg_name = to;
     msg.msg_namelen = sizeof(*to);
     msg.msg_control = NULL;
@@ -138,13 +230,27 @@ osi_NetReceive(osi_socket so, struct sockaddr_in *from, struct iovec *iov,
 {
     KERNEL_SPACE_DECL;
     struct msghdr msg;
-    int code;
+    int code, sockerr;
+    size_t esize;
     struct iovec tmpvec[RX_MAXWVECS + 2];
     struct socket *sop = (struct socket *)so;
 
     if (iovcnt > RX_MAXWVECS + 2) {
 	osi_Panic("Too many (%d) iovecs passed to osi_NetReceive\n", iovcnt);
     }
+#ifdef ADAPT_PMTU
+    while (1) {
+	sockerr=0;
+	esize = sizeof(sockerr);
+ 	TO_USER_SPACE();
+	sop->ops->getsockopt(sop, SOL_SOCKET, SO_ERROR, (char *)&sockerr,
+			   &esize);
+	TO_KERNEL_SPACE();
+	if (sockerr == 0)
+	   break;
+	handle_socket_error(so);
+    }
+#endif
     memcpy(tmpvec, iov, iovcnt * sizeof(struct iovec));
     msg.msg_name = from;
     msg.msg_iov = tmpvec;
@@ -158,6 +264,32 @@ osi_NetReceive(osi_socket so, struct sockaddr_in *from, struct iovec *iov,
     TO_KERNEL_SPACE();
 
     if (code < 0) {
+#ifdef AFS_LINUX26_ENV
+#ifdef CONFIG_PM
+	if (
+#ifdef PF_FREEZE
+	    current->flags & PF_FREEZE
+#else
+#if defined(STRUCT_TASK_STRUCT_HAS_TODO)
+	    !current->todo
+#else
+#if defined(STRUCT_TASK_STRUCT_HAS_THREAD_INFO)
+            test_ti_thread_flag(current->thread_info, TIF_FREEZE)
+#else
+            test_ti_thread_flag(task_thread_info(current), TIF_FREEZE)
+#endif
+#endif
+#endif
+	    )
+#ifdef LINUX_REFRIGERATOR_TAKES_PF_FREEZE
+	    refrigerator(PF_FREEZE);
+#else
+	    refrigerator();
+#endif
+	    set_current_state(TASK_INTERRUPTIBLE);
+#endif
+#endif
+
 	/* Clear the error before using the socket again.
 	 * Oh joy, Linux has hidden header files as well. It appears we can
 	 * simply call again and have it clear itself via sock_error().
@@ -176,26 +308,24 @@ osi_NetReceive(osi_socket so, struct sockaddr_in *from, struct iovec *iov,
 
     return code;
 }
-
+#ifdef EXPORTED_TASKLIST_LOCK
+extern rwlock_t tasklist_lock __attribute__((weak));
+#endif
 void
 osi_StopListener(void)
 {
     struct task_struct *listener;
     extern int rxk_ListenerPid;
+    extern struct task_struct *rxk_ListenerTask;
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
-    read_lock(&tasklist_lock);
-#endif
-    listener = find_task_by_pid(rxk_ListenerPid);
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
-    read_unlock(&tasklist_lock);
-#endif
-    while (rxk_ListenerPid) {
-	struct task_struct *p;
-
-	flush_signals(listener);
-	force_sig(SIGKILL, listener);
-	afs_osi_Sleep(&rxk_ListenerPid);
+    while (rxk_ListenerTask) {
+        if (rxk_ListenerTask) {
+	    flush_signals(rxk_ListenerTask);
+	    force_sig(SIGKILL, rxk_ListenerTask);
+	}
+	if (!rxk_ListenerTask)
+	    break;
+	afs_osi_Sleep(&rxk_ListenerTask);
     }
     sock_release(rx_socket);
     rx_socket = NULL;
