@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/volser/vsutils.c,v 1.18.8.3 2007/10/30 15:16:59 shadow Exp $");
+    ("$Header: /cvs/openafs/src/volser/vsutils.c,v 1.18.8.5 2008/04/14 20:25:52 shadow Exp $");
 
 #include <afs/stds.h>
 #include <string.h>
@@ -46,13 +46,14 @@ RCSID
 #include "volint.h"
 #include "lockdata.h"
 
+#include "vsutils_prototypes.h"
+
 struct ubik_client *cstruct;
 static rxkad_level vsu_rxkad_level = rxkad_clear;
 
 static void
-ovlentry_to_nvlentry(oentryp, nentryp)
-     struct vldbentry *oentryp;
-     struct nvldbentry *nentryp;
+ovlentry_to_nvlentry(struct vldbentry *oentryp,
+                     struct nvldbentry *nentryp)
 {
     register int i;
 
@@ -70,10 +71,9 @@ ovlentry_to_nvlentry(oentryp, nentryp)
     nentryp->flags = oentryp->flags;
 }
 
-static
-nvlentry_to_ovlentry(nentryp, oentryp)
-     struct nvldbentry *nentryp;
-     struct vldbentry *oentryp;
+static int
+nvlentry_to_ovlentry(struct nvldbentry *nentryp,
+                     struct vldbentry *oentryp)
 {
     register int i;
 
@@ -101,8 +101,8 @@ nvlentry_to_ovlentry(nentryp, oentryp)
 
 static int newvlserver = 0;
 
-VLDB_CreateEntry(entryp)
-     struct nvldbentry *entryp;
+int
+VLDB_CreateEntry(struct nvldbentry *entryp)
 {
     struct vldbentry oentry;
     register int code;
@@ -127,9 +127,8 @@ VLDB_CreateEntry(entryp)
     return code;
 }
 
-VLDB_GetEntryByID(volid, voltype, entryp)
-     afs_int32 volid, voltype;
-     struct nvldbentry *entryp;
+int
+VLDB_GetEntryByID(afs_int32 volid, afs_int32 voltype, struct nvldbentry *entryp)
 {
     struct vldbentry oentry;
     register int code;
@@ -154,9 +153,8 @@ VLDB_GetEntryByID(volid, voltype, entryp)
     return code;
 }
 
-VLDB_GetEntryByName(namep, entryp)
-     char *namep;
-     struct nvldbentry *entryp;
+int
+VLDB_GetEntryByName(char *namep, struct nvldbentry *entryp)
 {
     struct vldbentry oentry;
     register int code;
@@ -180,9 +178,8 @@ VLDB_GetEntryByName(namep, entryp)
     return code;
 }
 
-VLDB_ReplaceEntry(volid, voltype, entryp, releasetype)
-     afs_int32 volid, voltype, releasetype;
-     struct nvldbentry *entryp;
+int 
+VLDB_ReplaceEntry(afs_int32 volid, afs_int32 voltype, struct nvldbentry *entryp, afs_int32 releasetype)
 {
     struct vldbentry oentry;
     register int code;
@@ -212,11 +209,10 @@ VLDB_ReplaceEntry(volid, voltype, entryp, releasetype)
 }
 
 
-
-VLDB_ListAttributes(attrp, entriesp, blkentriesp)
-     VldbListByAttributes *attrp;
-     afs_int32 *entriesp;
-     nbulkentries *blkentriesp;
+int
+VLDB_ListAttributes(VldbListByAttributes *attrp,
+                    afs_int32 *entriesp,
+                    nbulkentries *blkentriesp)
 {
     bulkentries arrayEntries;
     register int code, i;
@@ -253,14 +249,13 @@ VLDB_ListAttributes(attrp, entriesp, blkentriesp)
     return code;
 }
 
-VLDB_ListAttributesN2(attrp, name, thisindex, nentriesp, blkentriesp,
-		      nextindexp)
-     VldbListByAttributes *attrp;
-     char *name;
-     afs_int32 thisindex;
-     afs_int32 *nentriesp;
-     nbulkentries *blkentriesp;
-     afs_int32 *nextindexp;
+int
+VLDB_ListAttributesN2(VldbListByAttributes *attrp,
+                      char *name,
+                      afs_int32 thisindex,
+                      afs_int32 *nentriesp,
+                      nbulkentries *blkentriesp,
+                      afs_int32 *nextindexp)
 {
     afs_int32 code;
 
@@ -286,8 +281,8 @@ struct cacheips {
 struct cacheips cacheips[GETADDRUCACHESIZE];
 int cacheip_index = 0;
 
-VLDB_IsSameAddrs(serv1, serv2, errorp)
-     afs_int32 serv1, serv2, *errorp;
+int
+VLDB_IsSameAddrs(afs_int32 serv1, afs_int32 serv2, afs_int32 *errorp)
 {
     register int code;
     ListAddrByAttributes attrs;
@@ -376,8 +371,7 @@ VLDB_IsSameAddrs(serv1, serv2, errorp)
   to get a rxkad_clear connection, simply don't call this.
 */
 void
-vsu_SetCrypt(cryptflag)
-     int cryptflag;
+vsu_SetCrypt(int cryptflag)
 {
     if (cryptflag) {
 	vsu_rxkad_level = rxkad_crypt;
@@ -391,13 +385,8 @@ vsu_SetCrypt(cryptflag)
   Get the appropriate type of ubik client structure out from the system.
 */
 afs_int32
-vsu_ClientInit(noAuthFlag, confDir, cellName, sauth, uclientp, secproc)
-     int noAuthFlag;
-     int (*secproc) ();
-     char *cellName;
-     struct ubik_client **uclientp;
-     char *confDir;
-     afs_int32 sauth;
+vsu_ClientInit(int noAuthFlag, char *confDir, char *cellName, afs_int32 sauth,
+               struct ubik_client **uclientp, int (*secproc)())
 {
     return ugen_ClientInit(noAuthFlag, confDir, cellName, sauth, uclientp, 
 			   secproc, "vsu_ClientInit", vsu_rxkad_level,
@@ -410,8 +399,7 @@ vsu_ClientInit(noAuthFlag, confDir, cellName, sauth, uclientp, secproc)
  * and return the result as <rname>.
  */
 int
-vsu_ExtractName(rname, name)
-     char rname[], name[];
+vsu_ExtractName(char rname[], char name[])
 {
     char sname[VOLSER_OLDMAXVOLNAME + 1];
     int total;
@@ -439,12 +427,8 @@ vsu_ExtractName(rname, name)
 
 /* returns 0 if failed */
 afs_uint32
-vsu_GetVolumeID(astring, acstruct, errp)
-     struct ubik_client *acstruct;
-     afs_int32 *errp;
-     char *astring;
+vsu_GetVolumeID(char *astring, struct ubik_client *acstruct, afs_int32 *errp)
 {
-    afs_uint32 value;
     char volname[VOLSER_OLDMAXVOLNAME + 1];
     struct nvldbentry entry;
     afs_int32 vcode = 0;
